#!/usr/bin/python
# Copyright (c) 2020, 2021 Oracle and/or its affiliates.
# This software is made available to you under the terms of the GPL 3.0 license or the Apache 2.0 license.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# Apache License v2.0
# See LICENSE.TXT for details.
# GENERATED FILE - DO NOT EDIT - MANUAL CHANGES WILL BE OVERWRITTEN


from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: oci_waas_custom_protection_rule_actions
short_description: Perform actions on a CustomProtectionRule resource in Oracle Cloud Infrastructure
description:
    - Perform actions on a CustomProtectionRule resource in Oracle Cloud Infrastructure
    - For I(action=change_compartment), moves a custom protection rule into a different compartment within the same tenancy. When provided, If-Match is checked
      against ETag values of the custom protection rule. For information about moving resources between compartments, see L(Moving Resources to a Different
      Compartment,https://docs.cloud.oracle.com/iaas/Content/Identity/Tasks/managingcompartments.htm#moveRes).
version_added: "2.9.0"
author: Oracle (@oracle)
options:
    custom_protection_rule_id:
        description:
            - The L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the custom protection rule. This number is generated when
              the custom protection rule is added to the compartment.
        type: str
        aliases: ["id"]
        required: true
    compartment_id:
        description:
            - The L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the compartment into which the resource should be moved. For
              information about moving resources between compartments, see L(Moving Resources to a Different
              Compartment,https://docs.cloud.oracle.com/iaas/Content/Identity/Tasks/managingcompartments.htm#moveRes).
        type: str
        required: true
    action:
        description:
            - The action to perform on the CustomProtectionRule.
        type: str
        required: true
        choices:
            - "change_compartment"
extends_documentation_fragment: [ oracle.oci.oracle ]
"""

EXAMPLES = """
- name: Perform action change_compartment on custom_protection_rule
  oci_waas_custom_protection_rule_actions:
    custom_protection_rule_id: "ocid1.customprotectionrule.oc1..xxxxxxEXAMPLExxxxxx"
    compartment_id: "ocid1.compartment.oc1..xxxxxxEXAMPLExxxxxx"
    action: change_compartment

"""

RETURN = """
custom_protection_rule:
    description:
        - Details of the CustomProtectionRule resource acted upon by the current operation
    returned: on success
    type: complex
    contains:
        id:
            description:
                - The L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the custom protection rule.
            returned: on success
            type: str
            sample: "ocid1.resource.oc1..xxxxxxEXAMPLExxxxxx"
        compartment_id:
            description:
                - The L(OCID,https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the custom protection rule's compartment.
            returned: on success
            type: str
            sample: "ocid1.compartment.oc1..xxxxxxEXAMPLExxxxxx"
        display_name:
            description:
                - The user-friendly name of the custom protection rule.
            returned: on success
            type: str
            sample: display_name_example
        description:
            description:
                - The description of the custom protection rule.
            returned: on success
            type: str
            sample: description_example
        mod_security_rule_ids:
            description:
                - The auto-generated ID for the custom protection rule. These IDs are referenced in logs.
            returned: on success
            type: list
            sample: []
        template:
            description:
                - The template text of the custom protection rule. All custom protection rules are expressed in ModSecurity Rule Language.
                - Additionally, each rule must include two placeholder variables that are updated by the WAF service upon publication of the rule.
                - "`id: {{id_1}}` - This field is populated with a unique rule ID generated by the WAF service which identifies a `SecRule`. More than one
                  `SecRule` can be defined in the `template` field of a CreateCustomSecurityRule call. The value of the first `SecRule` must be `id: {{id_1}}`
                  and the `id` field of each subsequent `SecRule` should increase by one, as shown in the example."
                - "`ctl:ruleEngine={{mode}}` - The action to be taken when the criteria of the `SecRule` are met, either `OFF`, `DETECT` or `BLOCK`. This field
                  is automatically populated with the corresponding value of the `action` field of the `CustomProtectionRuleSetting` schema when the `WafConfig`
                  is updated."
                - "*Example:*
                    ```
                    SecRule REQUEST_COOKIES \\"regex matching SQL injection - part 1/2\\" \\\\
                            \\"phase:2,                                                 \\\\
                            msg:'Detects chained SQL injection attempts 1/2.',        \\\\
                            id: {{id_1}},                                             \\\\
                            ctl:ruleEngine={{mode}},                                  \\\\
                            deny\\"
                    SecRule REQUEST_COOKIES \\"regex matching SQL injection - part 2/2\\" \\\\
                            \\"phase:2,                                                 \\\\
                            msg:'Detects chained SQL injection attempts 2/2.',        \\\\
                            id: {{id_2}},                                             \\\\
                            ctl:ruleEngine={{mode}},                                  \\\\
                            deny\\"
                    ```"
                - The example contains two `SecRules` each having distinct regex expression to match the `Cookie` header value during the second input analysis
                  phase.
                - For more information about custom protection rules, see L(Custom Protection
                  Rules,https://docs.cloud.oracle.com/Content/WAF/tasks/customprotectionrules.htm).
                - "For more information about ModSecurity syntax, see L(Making Rules: The Basic
                  Syntax,https://www.modsecurity.org/CRS/Documentation/making.html)."
                - For more information about ModSecurity's open source WAF rules, see L(Mod Security's OWASP Core Rule Set
                  documentation,https://www.modsecurity.org/CRS/Documentation/index.html).
            returned: on success
            type: str
            sample: template_example
        lifecycle_state:
            description:
                - The current lifecycle state of the custom protection rule.
            returned: on success
            type: str
            sample: CREATING
        time_created:
            description:
                - The date and time the protection rule was created, expressed in RFC 3339 timestamp format.
            returned: on success
            type: str
            sample: "2018-11-16T21:10:29Z"
        freeform_tags:
            description:
                - Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace.
                  For more information, see L(Resource Tags,https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
                - "Example: `{\\"Department\\": \\"Finance\\"}`"
            returned: on success
            type: dict
            sample: {'Department': 'Finance'}
        defined_tags:
            description:
                - Defined tags for this resource. Each key is predefined and scoped to a namespace.
                  For more information, see L(Resource Tags,https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
                - "Example: `{\\"Operations\\": {\\"CostCenter\\": \\"42\\"}}`"
            returned: on success
            type: dict
            sample: {'Operations': {'CostCenter': 'US'}}
    sample: {
        "id": "ocid1.resource.oc1..xxxxxxEXAMPLExxxxxx",
        "compartment_id": "ocid1.compartment.oc1..xxxxxxEXAMPLExxxxxx",
        "display_name": "display_name_example",
        "description": "description_example",
        "mod_security_rule_ids": [],
        "template": "template_example",
        "lifecycle_state": "CREATING",
        "time_created": "2018-11-16T21:10:29Z",
        "freeform_tags": {'Department': 'Finance'},
        "defined_tags": {'Operations': {'CostCenter': 'US'}}
    }
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.oracle.oci.plugins.module_utils import (
    oci_common_utils,
    oci_wait_utils,
)
from ansible_collections.oracle.oci.plugins.module_utils.oci_resource_utils import (
    OCIActionsHelperBase,
    get_custom_class,
)

try:
    from oci.waas import WaasClient
    from oci.waas.models import ChangeCustomProtectionRuleCompartmentDetails

    HAS_OCI_PY_SDK = True
except ImportError:
    HAS_OCI_PY_SDK = False


class CustomProtectionRuleActionsHelperGen(OCIActionsHelperBase):
    """
    Supported actions:
        change_compartment
    """

    @staticmethod
    def get_module_resource_id_param():
        return "custom_protection_rule_id"

    def get_module_resource_id(self):
        return self.module.params.get("custom_protection_rule_id")

    def get_get_fn(self):
        return self.client.get_custom_protection_rule

    def get_resource(self):
        return oci_common_utils.call_with_backoff(
            self.client.get_custom_protection_rule,
            custom_protection_rule_id=self.module.params.get(
                "custom_protection_rule_id"
            ),
        )

    def change_compartment(self):
        action_details = oci_common_utils.convert_input_data_to_model_class(
            self.module.params, ChangeCustomProtectionRuleCompartmentDetails
        )
        return oci_wait_utils.call_and_wait(
            call_fn=self.client.change_custom_protection_rule_compartment,
            call_fn_args=(),
            call_fn_kwargs=dict(
                custom_protection_rule_id=self.module.params.get(
                    "custom_protection_rule_id"
                ),
                change_custom_protection_rule_compartment_details=action_details,
            ),
            waiter_type=oci_wait_utils.NONE_WAITER_KEY,
            operation="{0}_{1}".format(
                self.module.params.get("action").upper(),
                oci_common_utils.ACTION_OPERATION_KEY,
            ),
            waiter_client=self.get_waiter_client(),
            resource_helper=self,
            wait_for_states=self.get_action_desired_states(
                self.module.params.get("action")
            ),
        )


CustomProtectionRuleActionsHelperCustom = get_custom_class(
    "CustomProtectionRuleActionsHelperCustom"
)


class ResourceHelper(
    CustomProtectionRuleActionsHelperCustom, CustomProtectionRuleActionsHelperGen
):
    pass


def main():
    module_args = oci_common_utils.get_common_arg_spec(
        supports_create=False, supports_wait=False
    )
    module_args.update(
        dict(
            custom_protection_rule_id=dict(aliases=["id"], type="str", required=True),
            compartment_id=dict(type="str", required=True),
            action=dict(type="str", required=True, choices=["change_compartment"]),
        )
    )

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)

    if not HAS_OCI_PY_SDK:
        module.fail_json(msg="oci python sdk required for this module.")

    resource_helper = ResourceHelper(
        module=module,
        resource_type="custom_protection_rule",
        service_client_class=WaasClient,
        namespace="waas",
    )

    result = resource_helper.perform_action(module.params.get("action"))

    module.exit_json(**result)


if __name__ == "__main__":
    main()
